from __future__ import print_function, absolute_import

from time import time
from txmsgpackrpc.server import MsgpackRPCServer
from txmsgpackrpc.factory import MsgpackServerFactory

from zope.interface import provider
from twisted.logger import Logger, ILogObserver
from twisted.internet import defer

from server_dao.mongo import MongoDAO
from server_dao.exception import NoSuchPeer


SERVER_PORT = 18080

@provider(ILogObserver)
class PrintingObserver:
    def __call__(self, event):
        print("[{level}] {text}".format(level=event['log_level'].name.upper(),
                                        text=formatEvent(event)))

class MongoRPCServerFactory(MsgpackServerFactory):
    """
    Overrides MsgpackServerFactory
    """
    log = Logger()

    def buildProtocol(self, addr):
        p = self.protocol(self, sendErrors=False) # Override to False
        return p

    def getRemoteMethod(self, protocol, methodName):
        # This is how you can get the ip address of the peer...
        # Maybe something ugly can be made so that it's passed to the method
        # when needed, like remote_register
        if methodName == 'beacon':
            # When registering we want to know from which ip address the peer
            # is connecting and so we patch the function with an extra parameter
            host = protocol.transport.getPeer().host
            base_function = getattr(self.handler, "remote_" + methodName)
            mod_function = lambda *args: base_function(host, *args)
            return mod_function
        elif methodName == 'register':
            # When registering we want to know from which ip address the peer
            # is connecting and so we patch the function with an extra parameter
            host = protocol.transport.getPeer().host
            base_function = getattr(self.handler, "remote_" + methodName)
            mod_function = lambda *args: base_function(host, *args)
            return mod_function
        else:
            return getattr(self.handler, "remote_" + methodName)

class MongoRPCServer(MsgpackRPCServer):
    log = Logger()

    def _error(self, error):
        import traceback
        self.log.error("{error}\n{traceback}",
                       error=error, traceback=traceback.format_exc(error))
    def __init__(self):
        self.dao = MongoDAO()

    def _response(self, status, body):
        if status == True:
            return {'status':'success','result':body}
        else:
            return {'status':'failure','error':body}

    @defer.inlineCallbacks
    def remote_beacon(self, host, peer_id):
        """
        This remote method gets special handling in getRemoteMethod() in the
        MongoRPCServerFactory class and thus receives the ip of the connecting
        peer
        """
        try:
            self.log.info("peer with id {peer_id} beaconed from host {host}",
                          peer_id=peer_id, host=host)
            result = yield self.dao.peer_beacon(peer_id, host, int(time()))
            self.log.info("beacon with result: {result}",result=result)
            defer.returnValue(self._response(True, result))
        except Exception as e:
            self._error(e)
            defer.returnValue(self._response(False, "error"))

    @defer.inlineCallbacks
    def remote_register(self, host, peer_id):
        """
        This remote method gets special handling in getRemoteMethod() in the
        MongoRPCServerFactory class and thus receives the ip of the connecting
        peer
        """
        try:
            self.log.info("peer with id {peer_id} registering from host {host}",
                          peer_id=peer_id, host=host)
            result = yield self.dao.register_peer(peer_id, host)
            #self.log.info("register with result: {result}",result=result)
            defer.returnValue(self._response(True, result))
        except Exception as e:
            self._error(e)
            defer.returnValue(self._response(False, "error"))


    @defer.inlineCallbacks
    def remote_get_active(self, peer_id):
        try:
            peers = yield self.dao.active_peers()
            defer.returnValue(self._response(True, peers))
        except Exception as e:
            self._error(e)
            defer.returnValue(self._response(False, "error"))

    @defer.inlineCallbacks
    def remote_get_tasks(self, peer_id):
        try:
            # no valid task generation yet...
            tasks = yield self.dao.get_tasks(peer_id)
            task_str=""
            for t in tasks:
                task_str += str(t)
            self.log.info("got tasks {tasks}", tasks=task_str)
            defer.returnValue(self._response(True, task_str))
        except NoSuchPeer as e:
            defer.returnValue(self._response(False, "nosuchpeer"))
        except Exception as e:
            self._error(e)
            defer.returnValue(self._response(False, "error"))


    @defer.inlineCallbacks
    def remote_upload_result(self, data):
        try:
            find = yield self.dao.find_peer(data['peer_id'])
            #self.log.info("{result} {count}", result=find[0], count=find.count())
            update = yield self.dao.write_result(data['peer_id'], data['result'])
            if update['status'] == 'success':
                defer.returnValue(self._response(True, update['status']))
            else:
                defer.returnValue(self._response(False, "Failed to upload."))
        except Exception as e:
            self._error(e)
            defer.returnValue(self._response(False, "error"))

class DataMuncher(object):
    log = Logger()

    def __init__(self):
        self.dao = MongoDAO()
        #from twisted.internet import task
        #self.tasks = [
        #        (task.LoopingCall(self.purge_old_results), 15),
        #        (task.LoopingCall(self.delegate_tasks), 10)
        #]

    def register(self):
        """ start tasks """
        pass
        #for task in self.tasks:
        #    task[0].start(task[1])

    @defer.inlineCallbacks
    def delegate_tasks(self):
        from random import randint
        peers = yield self.dao.get_peers()

        for peer in peers:
            self.log.info("Delegating tasks for peer {peer}", peer=peer['peer_id'])
            target = yield self.dao.get_random_peer()
            #self.log.info("target is {target}", target=target)
            if target is not None and peer['peer_id'] != target['peer_id']:
                #self.log.info("found target {target}", target=target['peer_id'])
                task = {
                    'task_name':'ping',
                    'task_id':randint(0,99999), # placeholder
                    'target_id':target['peer_id'],
                    'target_host':target['host']
                }
                self.log.info("New task {} ".format(task['task_id']) +
                              "for {}: ".format(peer['peer_id']) +
                              "ping to {}".format(task['target_id']))
                result = yield self.dao.write_task(peer['peer_id'], task)

    @defer.inlineCallbacks
    def purge_old_results(self):
        from time import time
        limit = time() - (30) # 30s ago

        self.log.info("Starting purge")
        peers = yield self.dao.get_peers()
        for peer in peers:
            self.log.info("purging results of peer {peer}", peer=peer['peer_id'])
            result = yield self.dao.purge_results_older_than(peer['peer_id'], limit)
            #self.log.info("{result}", result=result)


if __name__ == "__main__":
    log = Logger()

    from twisted.internet import reactor
    from twisted.logger import formatEvent, globalLogPublisher

    globalLogPublisher.addObserver(PrintingObserver())

    rpc_server = MongoRPCServer().getStreamFactory(MongoRPCServerFactory)

    muncher = DataMuncher()
    muncher.register()

    reactor.listenTCP(SERVER_PORT, rpc_server)
    log.info("Starting server on port %d..." % SERVER_PORT)
    reactor.run()



from __future__ import unicode_literals, absolute_import, print_function

import json

from cheesepilib.exceptions import UnsupportedResultType

class ResultParser(object):

	def __enter__(self):
		self.parse()
		return self

	def __exit__(self, exc_type, exc_value, traceback):
		pass

	@classmethod
	def fromFile(cls, filename):
		with open(filename) as fd:
			json_obj = json.load(fd)
		return cls.fromJson(json_obj)

	@classmethod
	def fromJson(cls, json_obj):
		from .PingResultParser import PingResultParser

		name = cls.get_taskname(json_obj)

		if name == 'ping': return PingResultParser(json_obj)
		else: raise UnsupportedResultType("Unknown task type '{}'.".format(name))

	@staticmethod
	def get_taskname(obj):
		return obj[0]['series'][0]['name']

	def parse(self):
		raise NotImplementedError(
		        "Abstract method 'parse' not implemented")

	def get_result_set():
		raise NotImplementedError(
		        "Abstract method 'get_result_set' not implemented")

	def write_to_db(self):
		raise NotImplementedError(
		        "Abstract method 'write_to_db' not implemented")



#!/usr/bin/python
# -*- coding: utf-8 -*-

import MySQLdb as mdb
import datetime

def main():
	host = "localhost"
	user = "urban"
	db = "sicspi"

	pw = raw_input("Please supply your password:\n")
	connectionDB = mdb.connect(host, user, pw, db)

	with connectionDB:
		cur = connectionDB.cursor()
		
		cur.execute("DELETE FROM Ping WHERE true")
		cur.execute("DELETE FROM Operation WHERE true")
		cur.execute("DELETE FROM Central_DNS WHERE true")
		cur.execute("DELETE FROM Connection WHERE true")
		cur.execute("DELETE FROM User WHERE true")
		cur.execute("DELETE FROM PI WHERE true")
		
		cur.execute("ALTER TABLE PI AUTO_INCREMENT = 1")
		cur.execute("ALTER TABLE Connection AUTO_INCREMENT = 1")
		cur.execute("ALTER TABLE Central_DNS AUTO_INCREMENT = 1")
		cur.execute("ALTER TABLE Operation AUTO_INCREMENT = 1")

	connectionDB.commit()
	connectionDB.close()


if __name__ == "__main__":
    main()



class DAO():

    # TODO: define parameters and return values explicitly with comments

    def __init__(self):
        pass

    def close(self):
        pass

    def peer_beacon(self, peer_id, host, last_seen):
        pass

    def active_peers(self, since):
        pass

    def register_peer(self, peer_id, host):
        pass

    def find_peer(self):
        pass

    def get_peers(self):
        pass

    def get_random_peer(self):
        pass

    def write_result(self, peer_id, result):
        pass

    def purge_results(self, peer_id):
        pass

    def purge_results_older_than(self, peer_id, timestamp):
        pass

    def write_task(self, peer_id, task):
        pass

    def _return_status(self, boolean):
        if boolean:
            return {'status':'success'}
        else:
            return {'status':'failure'}



""" Copyright (c) 2015, Swedish Institute of Computer Science
  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
	  * Redistributions of source code must retain the above copyright
		notice, this list of conditions and the following disclaimer.
	  * Redistributions in binary form must reproduce the above copyright
		notice, this list of conditions and the following disclaimer in the
		documentation and/or other materials provided with the distribution.
	  * Neither the name of The Swedish Institute of Computer Science nor the
		names of its contributors may be used to endorse or promote products
		derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE SWEDISH INSTITUTE OF COMPUTER SCIENCE BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Authors: ljjm@sics.se
Testers:
"""
import logging

# PyMongo
import MySQLdb

import dao

class DAO_mysql(dao.DAO):
	def __init__(self):
		try: # Get a hold of a MySQL connection
			self.conn = MySQLdb.connect("localhost", "measurement", "MP4MDb", "Measurement")
		except:
			msg = "Error: Connection to MySQL database failed! Ensure MySQL is running."
			logging.error(msg)
			print msg
			exit(1)

		if not self.conn:
			logging.error("MySQL database connnection failed")
			exit(1)

		#disable warnings
		cursor = self.conn.cursor()
		cursor.execute("""SET sql_notes = 0""")

		pingquery = """CREATE TABLE IF NOT EXISTS ping(ts TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
			sourceAddress TEXT, destinationDomain TEXT, destinationAddress TEXT,
			startingTime DATETIME, endingTime DATETIME, minimumRTT FLOAT,
			averageRTT FLOAT, maximumRTT FLOAT, packetLoss TEXT,
			ethernetMacAddress TEXT, currentMacAddress TEXT, packetSize INTEGER,
			numberOfPings INTEGER);"""
		cursor.execute(pingquery)

		httpquery = """CREATE TABLE IF NOT EXISTS httping(ts TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
			sourceAddress TEXT, destinationDomain TEXT, destinationAddress TEXT,
			startingTime DATETIME, endingTime DATETIME, minimumRTT FLOAT,
			averageRTT FLOAT, maximumRTT FLOAT, packetLoss TEXT,
			ethernetMacAddress TEXT, currentMacAddress TEXT, packetSize INTEGER,
			numberOfHttpings INTEGER);"""
		cursor.execute(httpquery)

		#enable warnings
		cursor.execute("""SET sql_notes = 1""")

	def close(self):
		self.conn.close()


	# operator level interactions
	def write_op(self, op_type, dic, binary=None):
		names = ""
		values = ""
		for key, value in dic.iteritems():
			names = names + key + ", "
			values = values + str(value) + ", "

		names = names[:-2]
		values = values[:-2]

		query = """INSERT INTO %s (%s) VALUES (%s)""" % (op_type, names, values)
		with self.conn:
			cursor = self.conn.cursor();
			cursor.execute(query)
			self.conn.commit()


	def read_op(self, op_type, timestamp=0, limit=100):
	#check last push and grab the rest?
		pass


	# user level interactions
	def read_user_attribute(self):
		pass


	def write_user_attribute(self, attribute, value):
		pass




import time
import os
import re
import logging
import socket

import cheesepi as cp
import Task

logger = cp.config.get_logger(__name__)

class Ping(Task.Task):

	# construct the process and perform pre-work
	def __init__(self, dao, spec):
		Task.Task.__init__(self, dao, spec)
		self.spec['taskname']    = "ping"
		if not 'landmark'    in self.spec: self.spec['landmark']    = "www.sics.se"
		if not 'ping_count'  in self.spec: self.spec['ping_count']  = 10
		if not 'packet_size' in self.spec: self.spec['packet_size'] = 64
		try:
			socket.gethostbyname(self.spec['landmark']) # we dont care, just populate the cache
		except:
			pass # record ping failure later

	# actually perform the measurements, no arguments required
	def run(self):
		logger.info("Pinging: %s @ %f, PID: %d" % (self.spec['landmark'], time.time(), os.getpid()))
		self.measure()

	# measure and record funtion
	def measure(self):
		start_time = cp.utils.now()
		op_output = self.perform(self.spec['landmark'], self.spec['ping_count'], self.spec['packet_size'])
		end_time = cp.utils.now()

		logger.debug(op_output)
		if op_output!=None: # we succeeded
			self.parse_output(op_output, self.spec['landmark'],
				start_time, end_time, self.spec['packet_size'], self.spec['ping_count'])
		self.dao.write_op(self.spec['taskname'], self.spec)


	#ping function
	def perform(self, landmark, ping_count, packet_size):
		packet_size -= 8 # change packet size to payload length!
		command = "ping -c %s -s %s %s"%(ping_count, packet_size, landmark)
		logging.info("Executing: "+command)
		logger.info(command)
		self.spec['return_code'], output = self.execute(command)

		if self.spec['return_code']==0:
			return output
		elif self.spec['return_code']==68:
			self.spec['error'] = "Unknown host"
		elif self.spec['return_code']==2:
			self.spec['error'] = "No response"
		return None

	#read the data from ping and reformat for database entry
	def parse_output(self, data, landmark, start_time, end_time, packet_size, ping_count):
		self.spec["start_time"]  = start_time
		self.spec["end_time"]    = end_time
		delays=[]

		lines = data.split("\n")
		first_line = lines.pop(0).split()
		self.spec["destination_domain"]  = first_line[1]
		self.spec["destination_address"] = re.sub("[()]", "", str(first_line[2]))

		delays = [-1.0] * ping_count# initialise storage
		for line in lines:
			if "time=" in line: # is this a PING return line?
				# does the following string wrangling always hold? what if not "X ms" ?
				# also need to check whether we are on linux-like or BSD-like ping
				if "icmp_req" in line: # BSD counts from 1
					sequence_num = int(re.findall('icmp_.eq=[\d]+ ',line)[0][9:-1]) -1
				elif "icmp_seq" in line: # Linux counts from 0
					sequence_num = int(re.findall('icmp_.eq=[\d]+ ',line)[0][9:-1])
				else:
					logging.error("ping parse error:"+line)
					exit(1)
				delay = re.findall('time=.*? ms',line)[0][5:-3]
				# only save returned pings!
				delays[sequence_num-1]=float(delay)
			elif "packet loss" in line:
				loss = re.findall('[\d]+% packet loss',line)[0][:-13]
				self.spec["packet_loss"] = float(loss)
			elif "min/avg/max/" in line:
				fields = line.split()[3].split("/")
				self.spec["minimum_RTT"] = float(fields[0])
				self.spec["average_RTT"] = float(fields[1])
				self.spec["maximum_RTT"] = float(fields[2])
				self.spec["stddev_RTT"]  = float(fields[3])

		self.spec['delays']     = str(delays)
		self.spec['uploaded']   = self.spec['packet_size'] * self.spec['ping_count']
		self.spec['downloaded'] = 8 * self.spec['ping_count']

if __name__ == "__main__":
	#general logging here? unable to connect etc
	dao = cp.config.get_dao()

	spec = {'landmark':'www.sics.se'}
	ping_task = Ping(dao, spec)
	ping_task.run()




import sys

sys.path.append("/usr/local/")
import Task

class Reschedule(Task.Task):

	# construct the process and perform pre-work
	def __init__(self, dao, spec):
		Task.Task.__init__(self, dao, spec)
		self.spec['taskname']    = "reschedule"

	def run(self):
		print "Warning: Reschedule tasks should not be run!"





from __future__ import unicode_literals, absolute_import, print_function

import os
import hashlib

from twisted.web import server
from twisted.web.resource import Resource
from twisted.internet import reactor, defer

UPLOAD_PATH = "/tmp/cheesepi/"

class UploadHandler(Resource):

	def __init__(self):
		if not os.path.exists(UPLOAD_PATH):
			os.makedirs(UPLOAD_PATH)

	def render_POST(self, request):

		filename = os.path.join(UPLOAD_PATH,
		                        request.args['filename'][0])

		# Check the hash
		md5_hash = request.args['md5_hash'][0]

		m = hashlib.md5()
		m.update(request.args['fileupload'][0])
		upload_hash = m.hexdigest()
		if md5_hash != upload_hash:
			print("HASH DOES NOT MATCH")
			# Do we care??

		# Write the file
		with open(filename, 'wb') as fd:
				fd.write(request.args['fileupload'][0])

		upload_size = os.stat(filename).st_size

		# Schedule processing of file one second later
		# NOTE: Maybe we'd like to chain together the completion of the
		#       write with the processing using a callback?
		from cheesepilib.server.processing.utils import process_upload
		reactor.callLater(1, process_upload, filename)

		response = b'Received upload of size %d bytes\n' % upload_size
		request.setHeader(b'Content-Length', len(response))

		request.write(response)

		request.finish()

		return server.NOT_DONE_YET



from __future__ import print_function, absolute_import

from twisted.internet import reactor
from twisted.logger import Logger, globalLogPublisher

from cheeselib.logger import PrintingObserver
from cheeselib.server.rpc import CheeseRPCServerFactory, CheeseRPCServer
from cheeselib.server.storage.mongo import MongoDAO

SERVER_PORT = 18080

log = Logger()

globalLogPublisher.addObserver(PrintingObserver())

dao = MongoDAO()

rpc_server = CheeseRPCServer(dao).getStreamFactory(CheeseRPCServerFactory)

reactor.listenTCP(SERVER_PORT, rpc_server)
log.info("Starting server on port %d..." % SERVER_PORT)
reactor.run()



#!/usr/bin/python
# -*- coding: utf-8 -*-import MySQLdb as mdb

import MySQLdb as mdb
import datetime

def main():

	connectionDB = mdb.connect("localhost", "urban", "basketphone", "sicspi")
	
	with connectionDB:
		cur = connectionDB.cursor()
		
		query = """insert ignore into PI (EthMAC) select distinct(EthernetMacAddress) from buffer.ping"""
		cur.execute(query)

		query = """insert ignore into Connection (PID, MAC) select distinct(PI.PID), buffer.ping.currentMacAddress from PI, buffer.ping where PI.EthMAC = buffer.ping.ethernetMacAddress group by PI.PID"""
		cur.execute(query)

		query = """insert ignore into Central_DNS(IP, Domain) select distinct(bp.destinationAddress), bp.destinationDomain from buffer.ping as bp"""
		cur.execute(query)

		query = """insert ignore into Operation (CID, Tool, StartTime, EndTime, Status) select CID, "Ping", bp.StartingTime, bp.EndingTime, "0" from Connection, PI, buffer.ping as bp where PI.EthMAC=bp.EthernetMacAddress and bp.currentMacAddress=Connection.MAC and Connection.PID = PI.PID"""
		cur.execute(query)

		query = """insert ignore into Ping (PID, OID, SA, Domain_ID, minRTT, avgRTT, maxRTT, ival, size, packet_loss, file_path) select PI.PID, OID, bp.sourceAddress, Central_DNS.Domain_ID, bp.minimumRTT, bp.averageRTT, bp.maximumRTT, bp.numberOfPings, bp.packetSize, bp.packetloss, "" from buffer.ping as bp, PI, Connection, Central_DNS, Operation where PI.EthMAC=bp.EthernetMacAddress and bp.currentMacAddress=Connection.MAC and Connection.PID = PI.PID and Central_DNS.Domain = bp.destinationDomain and Central_DNS.IP=bp.destinationAddress and Operation.CID = Connection.CID and Operation.StartTime=bp.StartingTime and Operation.EndTime=bp.EndingTime"""
		cur.execute(query)

	connectionDB.commit()
	connectionDB.close()

	#Purge the buffer
	connectionBuffer = mdb.connect("localhost", "urban", "basketphone", "buffer")
	with connectionBuffer:
		cur = connectionBuffer.cursor()

		query = """delete from ping"""
		cur.execute(query)
	
	connectionBuffer.commit()
	connectionBuffer.close()


if __name__ == "__main__":
    main()



import time
import os
import uptime
import logging
import socket
from subprocess import Popen, PIPE

import cheesepi as cp
import Task

logger = cp.config.get_logger(__name__)

class Status(Task.Task):

	# construct the process and perform pre-work
	def __init__(self, dao, spec={}):
		Task.Task.__init__(self, dao, spec)
		self.spec['taskname']    = "status"
		self.spec['downloaded'] = 0
		self.spec['uploaded']   = 0

	# actually perform the measurements, no arguments required
	def run(self):
		logger.info("Status @ %f, PID: %d" % (time.time(), os.getpid()))
		self.measure()

	#main measure funtion
	def measure(self):
		ethmac = cp.utils.get_MAC()
		self.spec['start_time'] = cp.utils.now()
		op_output  = self.measure_uptime()
		self.measure_storage()
		self.measure_temperature()
		parsed_output = self.parse_output(op_output, ethmac)
		self.dao.write_op("status", parsed_output)

	def measure_uptime(self):
		execute = "uptime"
		logging.info("Executing: "+execute)
		logger.debug(execute)
		result = Popen(execute ,stdin=PIPE, stdout=PIPE, stderr=PIPE, shell=False)
		ret = result.stdout.read()
		result.stdout.flush()
		return ret

	def measure_storage(self):
		st = os.statvfs('/')
		self.spec['available_kb'] = (st.f_bavail * st.f_frsize) / 1024
		fs_size = st.f_blocks * st.f_frsize
		fs_used = (st.f_blocks - st.f_bfree) * st.f_frsize
		self.spec['used_storage'] = fs_used / float(fs_size)

	def measure_temperature(self):
		temp = cp.utils.get_temperature()
		if temp!=None:
			self.spec['temperature'] = float(temp/1000.0)

	#read the data from ping and reformat for database entry
	def parse_output(self, data, ethmac):
		self.spec["ethernet_MAC"]  = ethmac
		self.spec["current_MAC"]   = cp.utils.get_MAC()
		self.spec["source_address"]= cp.utils.get_SA()
		# shakey
		self.spec["local_address"] = [(s.connect(('8.8.8.8', 80)), s.getsockname()[0], s.close()) for s in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]][0][1]

		fields = data.split()
		#self.spec["uptime"] = fields[2][:-1]
		#self.spec["uptime"] = re.search("up .*?,",data).group(0)[3:-1]
		self.spec["uptime"] = float(uptime.uptime() / (60*60))
		self.spec["load1"]  = float(fields[-3][:-1])
		self.spec["load5"]  = float(fields[-2][:-1])
		self.spec["load15"] = float(fields[-1])

		return self.spec


if __name__ == "__main__":
	#general logging here? unable to connect etc
	dao = cp.config.get_dao()

	status_task = Status(dao)
	status_task.run()




from __future__ import unicode_literals, absolute_import, print_function

import logging
import math
import tarfile
import hashlib

# Processing functions, untar, calculating stats etc...
# TODO some kind of logging??
# How do we know where the database is???

def process_upload(uploaded_file):
	from .ResultDataProcessor import ResultDataProcessor

	with ResultDataProcessor(uploaded_file) as data:
		data.process()

# TODO Maybe this should be baked into ResultDataProcessor
def untar(filename, destination):
	with tarfile.open(filename) as tar:
		tar.extractall(destination)

def md5_filehash(filepath):
	hasher = hashlib.md5()
	with open(filepath) as fd:
		hasher.update(fd.read())
	return hasher.hexdigest()

# This might be better to include from a well maintained library
def median(input_list):
	lst = sorted(input_list)
	if len(lst) < 1:
		return 0
	if len(lst) % 2 == 1:
		return lst[((len(lst)+1)/2)-1]
	else:
		return float(sum(lst[(len(lst)/2)-1:(len(lst)/2)+1]))/2.0

class StatObject(object):
	"""
	TODO maybe rename to something better

	Keeps track of mean value, variance and standard deviation incrementally
	while new data is supplied.
	"""
	log = logging.getLogger("cheesepi.server.processing.StatObject")

	_DEFAULT_ALPHA = 0.5

	@classmethod
	def fromDict(cls, dct):
		"""
		Parses a dict object which should contain the keys 'value'
		"""
		try:
			mean = dct['mean']
			variance = dct['variance']
			return cls(mean, variance)
		except (KeyError,TypeError) as e:
			cls.log.exception("{} while parsing dict.".format(e.__class__.__name__))
			return cls(0, 0)

	def toDict(self):
		return {
			'mean':self._mean,
			'variance':self._variance,
			'std_dev':self._std_dev,
		}

	def __init__(self, mean=0, variance=0, alpha=_DEFAULT_ALPHA):
		self._mean = mean
		self._variance = variance
		self._std_dev = math.sqrt(variance)
		self._alpha = self._DEFAULT_ALPHA

	def __repr__(self):
		return "StatObject({mean}, {variance}, {std_dev}, alpha={alpha})".format(
				mean=self._mean,
				variance=self._variance,
				std_dev=self._std_dev,
				alpha=self._alpha)

	def set_alpha(self, alpha):
		self._alpha = alpha

	def get_mean(self):
	    return self._mean
	def get_variance(self):
	    return self._variance
	def get_std_dev(self):
	    return self._std_dev
	def get_alpha(self):
	    return self._alpha

	def add_datum(self, new_datum, alpha=_DEFAULT_ALPHA):
		"""
		Add a new data point
		"""
		delta = new_datum - self._mean
		increment = alpha * delta

		self._mean = self._mean + increment
		self._variance = (1-alpha) * (self._variance + (delta * increment))
		self._std_dev = math.sqrt(self._variance)



#!/usr/bin/python
import MySQLdb
import warnings
import socket
import fcntl
import struct
def main():
	ethmac = getEthMAC()
	prefix = ethmac + "-"
	db = MySQLdb.connect("localhost", "measurement", "MP4MDb", "Measurement")
	curs=db.cursor()
	#The auto increment id from the table is going to be concatenated the main id in traceroute table
	#For example 04:7d:7b:36:99:79-001 to uniqly identify a traceroute
	traceroute_seq = """CREATE TABLE if not exists Traceroute_seq(
				id bigint(20) NOT NULL AUTO_INCREMENT PRIMARY KEY);"""
	traceroute = """CREATE TABLE if not exists Traceroute(
				ID VARCHAR(100) NOT NULL PRIMARY KEY, 
				ts TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP ,
				ethernetMacAddress TEXT, currentMacAddress TEXT, sourceAddress TEXT, 
				destinationDomain TEXT, destinationAddress TEXT, 
				startingTime DATETIME, endingTime DATETIME);"""
	hop = """CREATE TABLE if not exists Hop( 
				ID VARCHAR(100) NOT NULL, hopNumber INTEGER, packetNumber INTEGER,
				packetDomainAddress TEXT, packetDestinationAddress TEXT, RTT FLOAT,
				FOREIGN KEY (ID) REFERENCES Traceroute(ID) ON DELETE CASCADE);"""
	#This trigger is used to 
	createTrigger = """CREATE TRIGGER Trg_Traceroute_insert BEFORE INSERT ON Traceroute
			   FOR EACH ROW BEGIN
  				INSERT INTO Traceroute_seq VALUES (NULL);
  				SET NEW.ID = CONCAT('%s', LPAD(LAST_INSERT_ID(), length(LAST_INSERT_ID()), '0'));
			   END;""" %prefix
	#insertToTable = """INSERT INTO table1 (name) 
          #                             VALUES ('Biniam'), ('Urban');"""
	with warnings.catch_warnings():
                warnings.simplefilter("ignore")
		curs.execute(traceroute_seq)
		curs.execute(traceroute)
		curs.execute(hop)
		curs.execute(createTrigger)
		#curs.execute(insertToTable)
	db.commit()
	db.close()

#get the ethernet mac address of this device
def getEthMAC(ifname = "eth0"):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    info = fcntl.ioctl(s.fileno(), 0x8927,  struct.pack('256s', ifname[:15]))
    return ''.join(['%02x:' % ord(char) for char in info[18:24]])[:-1]


if __name__ == "__main__":
 main()



import time
import os

import cheesepi as cp
import Task

logger = cp.config.get_logger(__name__)

# https://github.com/sivel/speedtest-cli
import speedtest

class Throughput(Task.Task):

	# construct the process and perform pre-work
	def __init__(self, dao, spec):
		Task.Task.__init__(self, dao, spec)
		self.spec['taskname'] = "throughput"

	# actually perform the measurements, no arguments required
	def run(self):
		logger.info("Speedtest throughput: @ %f, PID: %d" % (time.time(), os.getpid()))
		self.measure()

	# measure and record funtion
	def measure(self):
		self.spec['start_time'] = cp.utils.now()
		try:
			op_output = speedtest.speedtest()
		except:
			logger.error("speedtest.py failed to run...")
			return
		self.spec['end_time']= cp.utils.now()
		logger.debug(op_output)

		parsed_output = self.parse_output(op_output)
		self.dao.write_op(self.spec['taskname'], parsed_output)

	#read the data and reformat for database entry
	def parse_output(self, data):
		self.spec['download_speed'] = data['download']
		self.spec['upload_speed']   = data['upload']
		self.spec['serverid'] = data['serverid']
		self.spec['ping']     = data['ping']
		# how much data was transferred? calcualted from speedtest.py
		self.spec['downloaded'] = 19100
		self.spec['uploaded']   = 750000
		return self.spec

if __name__ == "__main__":
	#general logging here? unable to connect etc
	dao = cp.config.get_dao()

	spec={}
	throughput_task = Throughput(dao, spec)
	throughput_task.run()



from __future__ import unicode_literals, absolute_import, print_function

import os
import shutil
import logging

from cheesepilib.server.parsing.ResultParser import ResultParser
from cheesepilib.exceptions import UnsupportedResultType
from .utils import untar, md5_filehash

class ResultDataProcessor(object):
	"""
	Encapsulates file handling and cleanup of processing result data.
	"""
	log = logging.getLogger("cheesepi.server.parsing.ResultDataProcessor")

	def __init__(self, filepath):
		"""
		Object which encapsulates the handling of a result dump received
		by the server. Should be initialized with the absolute path to a
		tar archive with the results.
		"""
		self._extracted = False
		self._filepath = filepath
		self._path = os.path.dirname(filepath)

		self._md5_hash = md5_filehash(filepath)
		self._extract_path = os.path.join(self._path, self._md5_hash)

	def __enter__(self):
		"""
		Extract archive.
		"""
		self.extract()
		return self

	def __exit__(self, exc_type, exc_value, traceback):
		"""
		Cleanup extracted files and delete the original tar archive.
		"""
		self.delete_extracted()
		self.delete()

	def get_hash(self):
		return self._md5_hash

	def extract(self):
		self.log.info("Extracting {} --> {}".format(
			self._filepath, self._extract_path))
		untar(self._filepath, self._extract_path)
		self._extracted = True

	def delete_extracted(self):
		if not self._extracted:
			raise Exception("Data not extracted.")

		self.log.info("Deleting folder {}".format(self._extract_path))
		shutil.rmtree(self._extract_path)
		self._extracted = False

	def process(self):
		if not self._extracted:
			raise Exception("Data not extracted.")

		self.log.info("Processing files in {}".format(self._extract_path))

		from cheesepilib.server.storage.mongo import MongoDAO
		from pprint import pformat

		dao = MongoDAO('localhost', 27017)

		# Process every file in the extracted folder
		files = [os.path.join(self._extract_path, f)
				for f in os.listdir(self._extract_path)]
		for filename in files:
			try:
				#parser = ResultParser.fromFile(filename)
				with ResultParser.fromFile(filename) as parser:
					results = parser.parse()
					peer_id = parser.get_peer_id()
					#self.log.info(peer_id)

					stats = dao.get_stats_set_for_results(peer_id, results)
					self.log.info("Fetched old stats")
					#self.log.info("Fetched:\n{}".format(pformat(stats.toDict())))

					stats.absorb_results(results)
					self.log.info("Absorbed new results")
					#self.log.info("Absorbed:\n{}".format(pformat(stats.toDict())))

					bulk_writer = dao.get_bulk_writer()

					bulk_writer = dao.bulk_write_stats_set(bulk_writer, peer_id, stats)

					result = bulk_writer.execute()
					self.log.info("Bulk wrote to database with result: {}".format(result))
					#parser.write_to_db()

				#from pprint import PrettyPrinter
				#printer = PrettyPrinter(indent=2)
				#printer.pprint(output)

			except UnsupportedResultType as e:
				# TODO This suppresses the full stack trace for the moment, but
				# should be removed once all parsers have been implemented. This
				# is here to declutter the log while developing
				self.log.warn("{}".format(e))
			except Exception as e:
				self.log.exception("Error parsing file {}".format(filename))

	def delete(self):
		"""
		We're done, delete all files.
		"""
		self.log.info("Deleting file {}".format(self._filepath))
		os.remove(self._filepath)



import time
import os
import platform
import re

import cheesepi as cp
import Task

logger = cp.config.get_logger(__name__)

class Traceroute(Task.Task):

	def __init__(self, dao, spec):
		Task.Task.__init__(self, dao, spec)
		self.spec['taskname'] = "traceroute"
                if not 'landmark'    in self.spec: self.spec['landmark']    = "www.sics.se"

	def run(self):
		logger.info("Tracerouting %s @ %f PID: %d" % (self.landmark, time.time(), os.getpid()))
		self.measure(self.landmark)

	def measure(self, landmark):
		#Extract the ethernet MAC address of the PI
		startTime = cp.utils.now()
		output    = self.perform(landmark)
		endTime   = cp.utils.now()
		#trc, hoplist = reformat(tracerouteResult, startTime, endTime)
		logger.debug(output)
		parsed = self.parse(output, startTime, endTime)
		parsed['uploaded']= 8 * 3 * len(parsed['hoplist'])
		parsed['uploaded']= parsed['downloaded']
		self.insertData(self.dao, parsed['traceroute'], parsed['hoplist'])

	#Execute traceroute function
	def perform(self, target):
		#traceroute command"
		command = "traceroute %s"%(target)
		self.spec['return_code'], output = self.execute(command)
		if self.spec['return_code']==0:
			return output
		return None

	def parse_null(self, hop_count):
		return {'hop_count': hop_count,
			'domain1': "*", 'domain2': "*", 'domain3': "*",
			'ip1'    : "*", 'ip2'    : "*", 'ip3'    : "*",
			'delay1': -1, 'delay2': -1, 'delay3': -1, }

	#############################
	# Parse Linux command
	#

	def parse(self, data, start_time, end_time):
		ret={}
		lines = data.split("\n")
		ret['traceroute'] = self.parse_destination(lines[0], start_time, end_time)
		hops=[]
		for line in lines[1:-1]:
			hop_count = int(line[:3].strip())
			hops.append(self.parse_hop(hop_count, line[4:]))
		logger.debug("hops: ",hops)
		ret['hops']=hops
		return ret

	def parse_destination(self, destination, start_time, end_time):
		traceroute = {}
		fields = destination.split()
		traceroute['domain']	 = fields[2]
		traceroute['ip']		 = fields[3][1:-1]
		traceroute['start_time'] = start_time
		traceroute['end_time']	 = end_time
		return traceroute

	def parse_hop(self, hop_count, host_line):
		"""This does not yet deal with network problems"""
		hop={'hop_count':hop_count}
		retry="1" # string accumulator
		for match in re.finditer(r"\*|(([\w\.-]+) \(([\d\.]+)\)  ([\d\.ms ]+) )", host_line):
			if match.group(0)=="*": # found a non response
				hop['domain'+retry]="*"
				hop['ip'+retry]="*"
				hop['delay'+retry]="-1"
			else: # some host reploiued N times
				for delay in match.group(4).split("ms"):
					hop['domain'+retry]= match.group(2)
					hop['ip'+retry]    = match.group(3)
					hop['delay'+retry] = delay
					retry = str(int(retry)+1) # inc but keep as string
			retry = str(int(retry)+1) # inc but keep as string
		return hop


	#########################
	## Mac traceroute
	########################
	def parse_mac(self, data):
		hops=[]
		lines = data.split()
		lines.pop(0)
		hop_count=-1
		while (len(lines)>0):
			line = lines.pop(0)
			hop_count = int(line[:3].strip())
			logger.debug(hop_count)
			host_line = line[4:] # extract everything after hopcount
			host_fields = host_line.split()
			if len(host_fields)==3:
				hops.extend(self.parse_null(hop_count))
			elif len(host_fields)==8: # the same host responds for each retry
				hop_entries = self.parse_hop_1host(hop_count,host_fields)
				hops.extend(hop_entries)
			elif len(host_fields)==4: # multiple hosts respond at this hop
				retry2 = lines.pop(0)[4:] #pop the next 2 lines
				retry3 = lines.pop(0)[4:]
				hop_entries = self.parse_hop_3host(hop_count,host_line, retry2, retry3)
				hops.extend(hop_entries)
		logger.debug(hops)
		return hops


	def parse_hop_1host(hop_count, host_fields):
		return {'hop_count': hop_count,
			'domain1': host_fields[0], 'domain2': host_fields[0], 'domain3': host_fields[0],
			'ip1'    : host_fields[1], 'ip2'    : host_fields[1], 'ip3'    : host_fields[1],
			'delay1' : host_fields[2], 'delay2' : host_fields[4], 'delay3' : host_fields[6],
			}

	def parse_hop_3host(hop_count, retry1, retry2, retry3):
		retry1_fields = retry1.split()
		retry2_fields = retry2.split()
		retry3_fields = retry3.split()
		return {'hop_count': hop_count,
			'domain1': retry1_fields[0], 'domain2': retry1_fields[0], 'domain3': retry1_fields[0],
			'ip1'	: retry1_fields[1], 'ip2'	: retry2_fields[1], 'ip3'	: retry3_fields[1],
			'delay1': retry1_fields[2], 'delay2': retry2_fields[2], 'delay3': retry3_fields[2],
			}
	###


	#insert the tracetoute results into the database
	def insertData(dao, traceroute, hoplist):
		logger.debug("Writting to the Traceroute table")
		traceroute_id = dao.write_op("traceroute", traceroute)

		for hop in hoplist:
			logger.debug(hop)
			#hop.traceroute = traceroute_id
			hop['traceroute_id'] = traceroute_id
			dao.write_op("traceroot_hop",hop)


#parses arguments
if __name__ == "__main__":
	if platform.system()=="Darwin":
		print "Seems to be Darwin OS (Mac), exiting..."
		exit(0)

	#general logging here? unable to connect etc
	config = cp.config.get_config()
	dao = cp.config.get_dao()

	spec = {'landmark':'www.sics.se'}
	traceroute_task = Traceroute(dao, spec)
	traceroute_task.run()




from __future__ import unicode_literals, absolute_import, print_function

import random
import math
import sys
import json

import numpy as np

from scipy.stats import gamma

class PingResultMocker(object):

	def __init__(self, shape=2, loc=10, scale=3, lossrate=0, seed=None):

		if seed is not None:
			np.random.seed(seed)

		if shape is None:
			shape = 2
		if loc is None:
			loc = 10
		if scale is None:
			scale = 3

		self._shape = shape
		self._loc = loc
		self._scale = scale
		self._generate_dist()

	def _generate_dist(self):
		self._dist = gamma(self._shape, loc=self._loc, scale=self._scale)

	def sample_n(self, n):
		samples = self._dist.rvs(size=n)
		if lossrate != 0:
			for i in range(0, len(samples)):
				if random.uniform(0,1) < lossrate:
					samples[i] = -1

		return samples


	def set_seed(self, seed):
		np.random.seed(seed)
	def set_shape(self, shape):
		self._shape = shape
		self._generate_dist()
	def set_loc(self, loc):
		self._loc = loc
		self._generate_dist()
	def set_scale(self, scale):
		self._scale = scale
		self._generate_dist()

	def get_mean(self):
		return self._dist.stats(moments='m').item()
	def get_variance(self):
		return self._dist.stats(moments='v').item()
	def get_std_dev(self):
		return math.sqrt(self._dist.stats(moments='v').item())
	def get_skew(self):
		return self._dist.stats(moments='s').item()
	def get_kurtosis(self):
		return self._dist.stats(moments='k').item()

class PingResultObjectConstructor(object):

	def __init__(self, peer_id):
		self._peer_id = peer_id
		self._results = []

	def add_result(self, data, target_id, destination_address):
		# TODO packet loss assumed 0 for now but should extend so we can model
		# that as well in the future
		max_rtt = 0
		min_rtt = sys.maxint
		stddev_rtt = 0
		average_rtt = 0
		packet_loss = 0
		ping_count = len(data)

		total_sum = 0
		square_sum = 0
		for d in data:
			# Make sure to catch packet loss before anything else
			if d < 0:
				packet_loss = packet_loss + 1
				continue

			if d < min_rtt:
				min_rtt = d
			if d > max_rtt:
				max_rtt = d
			total_sum = total_sum + d
			square_sum = square_sum + math.pow(d, 2)

		# NOTE We need to account for lost packets when calculating things
		success_count = ping_count - packet_loss

		variance = square_sum/success_count

		stddev_rtt = math.sqrt(variance)
		average_rtt = total_sum/success_count


		result = [
		    self._peer_id,
		    target_id,
		    None,
		    None,
		    str(data),
		    destination_address,
		    None,
		    None,
		    None,
		    None,
		    max_rtt,
		    min_rtt,
		    None,
		    packet_loss,
		    None,
		    None,
		    ping_count,
		    None,
		    None,
		    None,
		    stddev_rtt,
		    'ping',
		    average_rtt,
		    None,
		    None,
		]

		self._results.append(result)

	def construct(self):
		columns = [
		    "peer_id",
		    "target_id",
		    "time",
		    "cycle",
		    "delays",
		    "destination_address",
		    "destination_domain",
		    "downloaded",
		    "end_time",
		    "landmark",
		    "maximum_RTT",
		    "minimum_RTT",
		    "offset",
		    "packet_loss",
		    "packet_size",
		    "period",
		    "ping_count",
		    "sign",
		    "source",
		    "start_time",
		    "stddev_RTT",
		    "taskname",
		    "average_RTT",
		    "uploaded",
		    "version"
		]
		values = []
		for result in self._results:
			values.append(result)
		obj = [{'series':[{'values':values,'name':'ping','columns':columns}]}]
		return obj

if __name__ == "__main__":
	import argparse
	import ast
	from pprint import pformat

	parser = argparse.ArgumentParser()
	parser.add_argument('--peerid', type=str, default='1',
	                    help='the peer id the results belong to')
	parser.add_argument('--samplesize', type=int, default=10,
	                    help='number of samples for each result')
	parser.add_argument('--target', type=str, action='append',
	        help='the targets on the form: "{\'id\':id,\'ip\':ip}" with optional arguments \'shape\', \'loc\', \'scale\' and \'lossrate\' to modify the distribution')
	parser.add_argument('--seed', type=int, default=None,
	                    help='a random number seed')

	args = parser.parse_args()

	seed = args.seed
	proc = PingResultObjectConstructor(args.peerid)
	dist_stats = {}

	for t in args.target:
		dct = ast.literal_eval(t)
		shape = None
		loc = None
		scale = None
		lossrate = None
		if 'shape' in dct:
			shape = dct['shape']
		if 'loc' in dct:
			loc = dct['loc']
		if 'scale' in dct:
			scale = dct['scale']
		if 'lossrate' in dct:
			lossrate = dct['lossrate']

		prm = PingResultMocker(shape=shape, loc=loc, scale=scale,
		                       lossrate=lossrate, seed=seed)
		samples = prm.sample_n(args.samplesize)

		proc.add_result(list(samples), dct['id'], dct['ip'])

		dist = {
			'mean':prm.get_mean(),
			'variance':prm.get_variance(),
			'std_dev':math.sqrt(prm.get_variance()),
			'skew':prm.get_skew(),
			'kurt':prm.get_kurtosis(),
		}

		dist_stats[dct['id']] = dist

		# Make sure we get different sets from every target even if the
		# distribution is identical
		if seed is not None:
			seed = seed + 1

	obj = proc.construct()
	obj[0]['series'][0]['distribution_stats'] = dist_stats

	print(json.dumps(obj, indent=4, sort_keys=True))

	#import matplotlib.pyplot as plt
	#plt.hist(samples, 100)
	#plt.show()



import os
import re
import datetime
from mod_python import apache


NOW = str(datetime.datetime.utcnow().strftime("%s"))
DUMP_DIR="/var/www/html/dump"
if not os.path.exists(DUMP_DIR):
	os.makedirs(DUMP_DIR)

def index(req):
	if not 'file' in req.form or not req.form['file'].filename:
		return "Error: Please upload a file"

	ethmac   = "unset"
	password = "unset"
	if 'ethmac' in   req.form: ethmac   = req.form['ethmac']
	if 'password' in req.form: password = req.form['password']

	user = auth_user(ethmac,password)

	# Record which IP send this file
	ip = req.get_remote_host(apache.REMOTE_NOLOOKUP)

	return save_file(req.form['file'], ip, user)

def auth_user(mac, password):
	# shoulc check if this really is the user's MAC address

	# check Eth MAC is well-formed
	if not re.match("([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})",mac):
		return "unset"

	return mac

def save_file(fileitem, ip, user=None):
	# strip leading path from file name to avoid directory traversal attacks
	filename = NOW+".tgz"
	if user!=None:
		filename = user+"-"+filename
	#os.path.basename(fileitem.filename)

	# build absolute path to files directory
	host_dir = os.path.join(DUMP_DIR, ip)
	ensure_dir(host_dir)
	filepath = os.path.join(host_dir, filename)

	if os.path.exists(filepath):
		return "Error: file already exists"

	fd = open(filepath, 'wb')
	while 1:
        	chunk = fileitem.file.read(100000)
        	if not chunk: break
        	fd.write (chunk)
	return 'The file "%s" was uploaded successfully from  %s' % (filepath, ip)

def ensure_dir(path):
	if not os.path.exists(path):
		os.makedirs(path)




""" Copyright (c) 2015, Swedish Institute of Computer Science
  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
	  * Redistributions of source code must retain the above copyright
		notice, this list of conditions and the following disclaimer.
	  * Redistributions in binary form must reproduce the above copyright
		notice, this list of conditions and the following disclaimer in the
		documentation and/or other materials provided with the distribution.
	  * Neither the name of The Swedish Institute of Computer Science nor the
		names of its contributors may be used to endorse or promote products
		derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE SWEDISH INSTITUTE OF COMPUTER SCIENCE BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Authors: ljjm@sics.se
Testers:
"""

import sys
import logging
import hashlib
import json

import cheesepi as cp
import dao

logger = cp.config.get_logger(__name__)

# Influx module, use legacy on RaspberryPi Linux
try:
	from influxdb.influxdb08 import InfluxDBClient
	from influxdb.influxdb08.client import InfluxDBClientError
except AttributeError as e:
	msg =  "Problem importing Python InfluxDB module!\n"
	msg += "Either due to this computer not having a timezone set.\n"
	msg += "Use `raspi-config` > Internationalisation Options to set one.\n"
	msg += "Alternatively, install 'pandas' through pip, rather than apt."
	print msg
	logger.error(msg)
	sys.exit(1)

host     = "localhost"
port     = 8086
username = "root"
password = "root"
database = "cheesepi"

class DAO_influx(dao.DAO):
	def __init__(self):
		#logging.info("Connecting to influx: %s %s %s" % (username,password,database))
		try: # Get a hold of a Influx connection
			self.conn = InfluxDBClient(host, port, username, password, database)
		except Exception as e:
			msg = "Error: Connection to Influx database failed! Ensure InfluxDB is running. "+str(e)
			logging.error(msg)
			print msg
			cp.config.make_databases()
			exit(1)

	def extract_series(self, dic):
		if len(dic)==1 and dic[0]['name']=='list_series_result':
			# mac version
			return [s[1] for s in dic[0]['points']]
		elif len(dic)>1 and dic[0]['points']==[]:
			# raspberry pi version
			return [s['name'] for s in dic]
		else:
			print "Error: parsing series list"


	def dump(self, since=-1):
		try:
			series_list = self.conn.query("list series")
			#series_list = self.conn.get_list_series()
		except Exception as e:
			msg = "Problem connecting to InfluxDB when listing series: "+str(e)
			print msg
			logging.error(msg)
			exit(1)
		print series_list
		series = self.extract_series(series_list)
		print series
		# maybe prune series list?
		dumped_db = {}
		for series_name in series:
			print series_name
			dumped_series = self.conn.query('select * from %s where time > %d ;' % (series_name,since*1000) )
			#print dumped_series
			dumped_db[series_name] = json.dumps(dumped_series)
		return dumped_db

	def format09(self,table,dic):
		#print [{"measurement":table,"fields":dic}]
		return [{'measurement':table,"database": "cheesepi","fields":dic,"tags": {"source":"dao"} }]
		#return json_body

	def format08(self, table, dic):
		for k in dic.keys():
				dic[k]=dic[k]
		#json_dic = [{"name":table, "columns":dic.keys(), "points":[dic.values()]}]
		json_str = '[{"name":"%s", "columns":%s, "points":[%s]}]' % (table,json.dumps(dic.keys()),json.dumps(dic.values()))
		#json_str = '[{"name":"ping", "columns":["test"], "points":["value"]}]'
		return json_str

	def slurp(self, op_type, points):
		"""Short cut to database write, useful for bulk writes"""
		#self.conn.write_points(op_type, points, batch_size=50)
		pass

	# Operator interactions
	def write_op(self, op_type, dic, binary=None):
		if not self.validate_op(op_type):
			logging.warning("Operation of type %s not valid: " % (op_type, str(dic)))
			return
		#if binary!=None:
		#	 # save binary, check its not too big
		#	 dic['binary'] = bson.Binary(binary)
		config = cp.config.get_config()
		dic['version'] = config['version']
		md5 = hashlib.md5(config['secret']+str(dic)).hexdigest()
		dic['sign']    = md5

		json = self.format08(op_type, dic)
		print "Saving %s Op: %s" % (op_type, json)
		try:
			return self.conn.write_points(json)
		except Exception as e:
			msg = "Database Influx "+op_type+" Op write failed! "+str(e)
			logging.error(msg)
			print msg
			#cp.config.make_databases()
			#exit(1)
			raise IOError(msg)
		return id

	def read_op(self, op_type, timestamp=0, limit=100):
		op = self.conn.query('select * from '+op_type+' limit 1;')
		return op



	## User level interactions
	# Note that assignments are not deleted, but the most recent assignemtn
	# is always returned
	def read_user_attribute(self, attribute):
		try:
			result = self.conn.query('select %s from user limit 1;' % attribute)
			if result==[]:
				return -1
			column_index = result[0]['columns'].index(attribute)
			value = result[0]['points'][0][column_index]
		except InfluxDBClientError:
			#msg = "Problem connecting to InfluxDB: "+str(e)
			#print msg
			#logging.error(msg)
			return -1
		except Exception as e:
			msg = "Problem connecting to InfluxDB: "+str(e)
			print msg
			exit(1)
		return value

	def write_user_attribute(self, attribute, value):
		# check we dont already exist
		try:
			print "Saving user attribute: %s to %s " % (attribute, value)
			json = self.format08("user", {attribute:value})
			return self.conn.write_points(json)
		except Exception as e:
			msg = "Problem connecting to InfluxDB: "+str(e)
			print msg
			logging.error(msg)
			exit(1)






from __future__ import unicode_literals, absolute_import, print_function

import heapq

from cheesepilib.server.storage.mongo import MongoDAO
from .Scheduler import Scheduler

class PingScheduler(Scheduler):

	def __init__(self, peer_id):
		self.dao = MongoDAO('localhost', 27017)
		self._peer_id = peer_id

	def get_schedule(self, num=1):
		schedule = []

		stats = self.dao.get_all_stats(self._peer_id)

		from pprint import pformat

		#print(pformat(stats.toDict()))

		priority_sorted_targets = []

		for s in stats:
			#print(pformat(s.toDict()))
			delay_variance = s.get_delay().get_variance()
			target = s.get_target()
			#print(target)
			#print(delay_variance)
			heapq.heappush(priority_sorted_targets, (-delay_variance, target))

		#print(priority_sorted_targets)

		for i in range(0, min(num,len(priority_sorted_targets))):
			target = heapq.heappop(priority_sorted_targets)
			schedule.append(target[1])

		return schedule


if __name__ == "__main__":
	import argparse
	import cheesepilib.server.utils as utils

	utils.init_logging()

	parser = argparse.ArgumentParser()
	parser.add_argument('--id', type=str, default=None)
	parser.add_argument('--num', type=int, default=1)

	args = parser.parse_args()

	ps = PingScheduler(args.id)

	schedule = ps.get_schedule(num=args.num)
	if len(schedule) == 0:
		print("Nothing scheduled")
	else:
		for target in schedule:
			print(target.get_uuid())



#!/usr/bin/python

import os
import datetime
import MySQLdb as mdb


def main():
	connectionDB = mdb.connect("localhost", "urban", "basketphone", "sicspi")
	cur = connectionDB.cursor()
        listOfPIs = pi_List(connectionDB, cur)
        pis_Not_Send_Data(listOfPIs,connectionDB, cur)
	connectionDB.close()
#A method that list out pis who do not send data today
def pis_Not_Send_Data(listOfPIs,connectionDB, cur):
        now = datetime.datetime.now()
        now = now.strftime("%Y-%m-%d")
        now = datetime.datetime.strptime(now, "%Y-%m-%d")
        #The list Pis which do not send data will be save in a file
        myfile = open("ListPIs.txt",'a+')
        myfile.write("\nPis which do not send data today: " + str(now))
        pisNotSendData = []
	print "Start looking for Pis who do not send data"
        #Iterating on the registered Pis
        for pi in listOfPIs:
                #Retrieves the starting time of the last data send by the PI  
		selectQuery = """SELECT StartTime FROM Operation,Ping,PI 
				 WHERE Operation.OID = Ping.OID AND Ping.PID = PI.PID AND 
                                 PI.EthMAC = '%s' ORDER BY Operation.StartTime DESC LIMIT 1;"""%str(pi[0])
                cur.execute(selectQuery)
		startTime = cur.fetchall()
		connectionDB.commit()
		startTime = startTime[0][0].strftime("%Y-%m-%d")
		startTime = datetime.datetime.strptime(startTime, "%Y-%m-%d")
		difference = now - startTime
		print difference
		#If the starting time is not today
		if (difference.days > 0):
			pisNotSendData.append(str(pi[0])) 
        if pisNotSendData:
		for pi in pisNotSendData:
                        myfile.write("\n" + str(pi))
        myfile.close()
	print "Writes the list of Pis who do not send data in the file"

#A metho that returns list of registerd pis in the central server
def pi_List(connectionDB, cur):
        selectQuery = """SELECT EthMAC from PI"""
        cur.execute(selectQuery)
        rows = cur.fetchall()
        connectionDB.commit()
        #connectionDB.close()
        return rows


if __name__ == "__main__":
    main()




from __future__ import unicode_literals, absolute_import

from .statistics import Statistics
from cheesepilib.server.processing.utils import StatObject
from cheesepilib.server.storage.models.target import Target

class PingStatistics(Statistics):

	@classmethod
	def fromDict(cls, dct):
		p = PingStatistics()
		p._target = Target.fromDict(dct['target'])
		p._delay = StatObject.fromDict(dct['delay'])
		p._average_delay = StatObject.fromDict(dct['average_delay'])
		p._average_median_delay = StatObject.fromDict(dct['average_median_delay'])
		p._average_packet_loss = StatObject.fromDict(dct['average_packet_loss'])
		p._all_time_min_rtt = dct['all_time_min_rtt']
		p._all_time_max_rtt = dct['all_time_max_rtt']
		p._total_packet_loss = dct['total_packet_loss']
		p._total_probe_count = dct['total_probe_count']

		return p

	def __init__(self, target=None):
		self._target = target
		self._delay = StatObject(0,0)
		self._average_delay = StatObject(0,0)
		self._average_median_delay = StatObject(0,0)
		self._average_packet_loss = StatObject(0,0)
		self._all_time_min_rtt = 999999999
		self._all_time_max_rtt = 0
		self._total_packet_loss = 0
		self._total_probe_count = 0

	def toDict(self):
		return {
			'task_name':'ping',
			'target':self._target.toDict(),
			'delay':self._delay.toDict(),
			'average_delay':self._average_delay.toDict(),
			'average_median_delay':self._average_median_delay.toDict(),
			'average_packet_loss':self._average_packet_loss.toDict(),
			'all_time_min_rtt':self._all_time_min_rtt,
			'all_time_max_rtt':self._all_time_max_rtt,
			'total_packet_loss':self._total_packet_loss,
			'total_probe_count':self._total_probe_count,
		}

	def get_type(self):
		return 'ping'

	def get_target(self):
		return self._target

	def get_delay(self):
		return self._delay
	def get_average_delay(self):
		return self._average_delay
	def get_average_median_delay(self):
		return self._average_median_delay
	def get_average_packet_loss(self):
		return self._average_packet_loss
	def get_all_time_min_rtt(self):
		return self._all_time_min_rtt
	def get_all_time_max_rtt(self):
		return self._all_time_max_rtt
	def get_total_packet_loss(self):
		return self._total_packet_loss
	def get_total_probe_count(self):
		return self._total_probe_count

	def absorb_result(self, result):
		from cheesepilib.server.processing.utils import median

		assert result.get_taskname() == 'ping'

		self._total_probe_count = self._total_probe_count + result.get_probe_count()
		self._total_packet_loss = self._total_packet_loss + result.get_packet_loss()
		self._all_time_min_rtt = min(self._all_time_min_rtt, result.get_min_rtt())
		self._all_time_max_rtt = max(self._all_time_max_rtt, result.get_max_rtt())

		# If we want to calculate the median we need a sequence without lost
		# packets
		pure_sequence = []
		for d in result.get_delay_sequence():
			# We need to ignore lost packets
			if d > 0:
				self._delay.add_datum(d)
				pure_sequence.append(d)

		sequence_median = median(pure_sequence)
		#self.log.info("MEDIAN {}".format(sequence_median))
		self._average_median_delay.add_datum(sequence_median)

		self._average_delay.add_datum(result.get_avg_rtt())
		#self.log.info("PACKET_LOSS {}".format(result.get_packet_loss()))
		#self.log.info("PROBE_COUNT {}".format(result.get_probe_count()))
		self._average_packet_loss.add_datum(float(result.get_packet_loss())/float(result.get_probe_count()))



#!/usr/bin/env python

# Placeholder script for a test suite that confirms the CheesePi distribution sitting
# in the current version of the repository is valid and has no obvious problems.
# It will return and error code (>0) if there is a problem, and 0 if things appear ok.

import sys


# seems ok
sys.exit(0)



""" Copyright (c) 2015, Swedish Institute of Computer Science
  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
	  * Redistributions of source code must retain the above copyright
		notice, this list of conditions and the following disclaimer.
	  * Redistributions in binary form must reproduce the above copyright
		notice, this list of conditions and the following disclaimer in the
		documentation and/or other materials provided with the distribution.
	  * Neither the name of The Swedish Institute of Computer Science nor the
		names of its contributors may be used to endorse or promote products
		derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE SWEDISH INSTITUTE OF COMPUTER SCIENCE BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Authors: ljjm@sics.se
Testers:
"""

import sys
import logging
import hashlib
import json
import traceback
from requests.exceptions import ConnectionError

import cheesepi as cp
import dao

logger = cp.config.get_logger(__name__)

try:
	from influxdb import InfluxDBClient
	from influxdb.client import InfluxDBClientError
except AttributeError as e:
	msg =  "Problem importing Python InfluxDB module!\n"
	msg += "Either due to this computer not having a timezone set.\n"
	msg += "Use `raspi-config` > Internationalisation Options to set one.\n"
	msg += "Alternatively, install 'pandas' through pip, rather than apt."
	logger.error(msg)
	sys.exit(1)

host     = "localhost"
port     = 8086
username = "root"
password = "root"
database = "cheesepi"

class DAO_influx(dao.DAO):
	def __init__(self):
		logger.info("Connecting to influx: %s %s %s" % (username,password,database))
		try: # Get a hold of a Influx connection
			self.conn = InfluxDBClient(host, port, username, password, database)
		except Exception as e:
			msg = "Error: Connection to Influx database failed! Ensure InfluxDB is running. "+str(e)
			logger.error(msg)
			logger.exception(e)
			cp.config.make_databases()
			exit(1)


	def dump(self, since=-1):
		try:
			series = self.conn.get_list_series()
		except Exception as e:
			msg = "Problem connecting to InfluxDB when listing series: "+str(e)
			logger.error(msg)
			logger.exception(e)
			exit(1)

		# maybe prune series?
		dumped_db = {}
		for s in series:
			series_name = s['name']
			logger.info(series_name)
			dumped_series = self.conn.query('select * from %s where time > %d limit 5;' % (series_name,0*1000) )
			logger.info(dumped_series.raw['results'])
			dumped_db[series_name] = json.dumps(dumped_series.raw['results'])
		return dumped_db



	def format09(self,table,dic):
		#print [{"measurement":table,"fields":dic}]
		return [{'measurement':table, "database":"cheesepi", "fields":dic, "tags":{"source":"dao"} }]
		#return json_body


	def format08(self, table, dic):
		for k in dic.keys():
				dic[k]=dic[k]
		#json_dic = [{"name":table, "columns":dic.keys(), "points":[dic.values()]}]
		#json_str = '[{"name":"%s", "columns":%s, "points":[%s]}]' % (table,json.dumps(dic.keys()),json.dumps(dic.values()))
		json_str = '[{"measurement":"%s", "fields":%s, ' % (table,"")
		#json_str = '[{"name":"ping", "columns":["test"], "points":["value"]}]'
		return json_str


	# Operator interactions
	def write_op(self, op_type, dic, binary=None):
		if not self.validate_op(op_type):
			logger.warning("Operation of type %s not valid: " % (op_type, str(dic)))
			return
		#if binary!=None:
		#	 # save binary, check its not too big
		#	 dic['binary'] = bson.Binary(binary)
		config = cp.config.get_config()
		dic['version'] = config['version']
		md5 = hashlib.md5(config['secret']+str(dic)).hexdigest()
		dic['sign']    = md5

		points=self.format09(op_type, dic)
		logger.debug(points)
		from pprint import pformat
		logger.info("Saving %s Op:\n%s" % (op_type, pformat(points)))
		result = None
		try:
			result = self.conn.write_points(points)
		except InfluxDBClientError as e:
			if e.code==204: # success!
				return True
			traceback.print_exc()
		except ConnectionError as e:
			logger.error("Database connection error, is the database server running?")
			return None
		except Exception as e:
			msg = "Database Influx "+op_type+" Op write failed! "+str(e)
			logger.error(msg)
			logger.exception(e)
			#cheesepi.config.make_databases()
			#exit(1)
			return None
		return result

	def read_op(self, op_type, timestamp=0, limit=100):
		op = self.conn.query('select * from '+op_type+' limit 1;')
		return op



	## User level interactions
	# Note that assignments are not deleted, but the most recent assignemtn
	# is always returned
	def read_user_attribute(self, attribute):
		try:
			result = self.conn.query('select %s from user limit 1;' % attribute)
			if result==[]:
				return -1
			column_index = result[0]['columns'].index(attribute)
			value = result[0]['points'][0][column_index]
		except InfluxDBClientError:
			#msg = "Problem connecting to InfluxDB: "+str(e)
			#print msg
			#logger.error(msg)
			return -1
		except Exception as e:
			msg = "Problem connecting to InfluxDB: "+str(e)
			logger.error(msg)
			logger.exception(e)
			exit(1)
		return value

	def write_user_attribute(self, attribute, value):
		# check we dont already exist
		try:
			logger.info("Saving user attribute: %s to %s " % (attribute, value))
			#json = self.to_json("user", {attribute:value})
			json = self.toFormat("user", {attribute:value})
			logger.debug(json)
			return self.conn.write_points(json)
		except Exception as e:
			msg = "Problem connecting to InfluxDB: "+str(e)
			logger.error(msg)
			logger.exception(e)
			exit(1)




#!/usr/bin/env python

from cStringIO import StringIO
import requests
import tarfile
import sys

install_dir = "/usr/local"
print "Downloading a CheesePi install into "+install_dir+"..."

# Location of the most recent release of the CheesePi code
code_url = "http://cheesepi.sics.se/files/cheesepi.tar.gz"

response = None
try:
    response = requests.head(url=code_url)
except Exception as e:
    print "Error: Could not make request to CheesePi server "+code_url+": "+str(e)
    exit(1)

if response.status_code!=200:
    print "Error: file %s was not available on server" % code_url
    exit(1)

lastmodified = response.headers['last-modified']
#print lastmodified

# if we have downloaded since it was updated, do nothing
response = requests.get(code_url)

fd = StringIO(response.content)
tfile = tarfile.open(mode="r:gz", fileobj=fd)

try:
	# should actually do this into /usr/local (or the correct cheesepi directory)
	tfile.extractall(install_dir)

	sys.path.append(install_dir)
	import cheesepi
	# record that we have just updated the code
	cheesepi.config.set_last_updated()
except OSError:
	print "Error: Can't untar the .tar.gz, you probably do not have permission, try sudo"
	exit(1)



import time
import os
import logging
from subprocess import Popen, PIPE

import cheesepi as cp
import Task

logger = cp.config.get_logger(__name__)

class iPerf(Task.Task):

	# construct the process and perform pre-work
	def __init__(self, dao, spec):
		Task.Task.__init__(self, dao, spec)
		self.spec['taskname'] = "iperf"
		if not 'landmark' in spec: self.spec['landmark'] = "iperf.eenet.ee"
		if not 'port' in spec: self.spec['port'] = 5201


	# actually perform the measurements, no arguments required
	def run(self):
		logger.info("iPerfing: %s:%d @ %f, PID: %d" % (self.spec['landmark'], self.spec['port'], time.time(), os.getpid()))
		self.measure()

	# measure and record funtion
	def measure(self):
		self.spec['start_time'] = cp.utils.now()
		op_output  = self.perform(self.spec['landmark'], self.spec['port'])
		self.spec['end_time']   = cp.utils.now()
		logger.debug(op_output)
		parsed_output = self.parse_output(op_output)
		self.dao.write_op(self.spec['taskname'], parsed_output)

	#ping function
	def perform(self, landmark, port):
		# an ARM version of this is located at 'client/tools/iperf3'
		execute = "iperf3 -yc -f k -p %d -c %s"%(port, landmark)
		print execute
		logging.info("Executing: "+execute)
		logger.debug(execute)
		result = Popen(execute ,stdin=PIPE, stdout=PIPE, stderr=PIPE, shell=True)
		ret = result.stdout.read()
		result.stdout.flush()
		return ret

	#read the data from ping and reformat for database entry
	def parse_output(self, data):

		lines = data.split("\n")
		fields = lines[0].split(',')
		self.spec['bandwidth'] = int(fields[-1])
		self.spec['transfer']  = int(fields[-2])
		return self.spec


if __name__ == "__main__":
	#general logging here? unable to connect etc
	dao = cp.config.get_dao()

	# Public servers https://iperf.fr/iperf-servers.php
	spec = {'landmark':"iperf.eenet.ee",
		'port':5201,
	}
	iperf_task = iPerf(dao, spec)
	iperf_task.run()





# The Root Exception
class CheesePiLibException(Exception):
    pass


# Peer Exceptions
class CheesePiException(CheesePiLibException):
    pass


# Server Exceptions
class CheesePiServerException(CheesePiLibException):
    pass

class ServerDaoError(CheesePiServerException):
    pass

class NoSuchPeer(ServerDaoError):
    pass

class UnsupportedResultType(CheesePiServerException):
    pass

class UnsupportedTargetType(CheesePiServerException):
    pass



#!/usr/bin/env python

import os
import logging
import cherrypy


#logger = logging.getLogger(__name__)
#logger.setLevel(logging.ERROR)

cherrypy.log.error_log.setLevel(30)

resultLimit = 5 # number of results per page
serveroot = os.path.dirname(os.path.realpath(__file__))
confpath  = os.path.join(serveroot,'cherrypy.conf')
#print "Webserver root: "+serveroot

#dao = cp.config.get_dao()

class Root:
	def index(self):
		raise cherrypy.HTTPRedirect("/dashboard")
		return
	index.exposed = True

class Dynamic:
	def index(self, **params):
		cherrypy.response.headers["Content-Type"]  = "application/json"
		return '{[{"value":10},{"value":15}]}'
		#return '{["value":1],["value":2]}'
		#return dao.get_op("ping")
	index.exposed = True

def setup_server():
	root = Root()
	root.data = Dynamic()
	config = {
		'global': {
			'environment': 'embedded',
			'log.screen': False,
		},
		'/dashboard': {
			'log.screen': False,
			'tools.staticdir.on': True,
			'tools.staticdir.root': serveroot,
			'tools.staticdir.dir': 'dashboard',
			'tools.staticdir.index': 'index.html',
		},
		}
	cherrypy.log.screen = False
	cherrypy.tree.mount(root, config=config)
	cherrypy.config.update({ 'server.socket_host':'0.0.0.0', 'server.socket_port':8080, })
	try:
		cherrypy.server.start()
	except IOError as e:
		msg = "Error: Can't start server, port probably already in use: "+str(e)
		print msg
		logging.error(msg)

if __name__ == "__main__":
	setup_server()







#!/usr/bin/python
# -*- coding: utf-8 -*-

import MySQLdb as mdb

con = mdb.connect('localhost', 'urban', 'basketphone', 'sicspi')

with con:
	
	cur = con.cursor()

	#cur.execute("DROP TABLE IF EXISTS Ping")
        #cur.execute("DROP TABLE IF EXISTS User")
	#cur.execute("DROP TABLE IF EXISTS Operation")
	#cur.execute("DROP TABLE IF EXISTS Connection")
	#cur.execute("DROP TABLE IF EXISTS PI")
	#cur.execute("DROP TABLE IF EXISTS Central_DNS")

#=====================================================================
	#warnings off
	cur.execute("""SET sql_notes = 0""")
#=====================================================================

	cur.execute("""CREATE TABLE IF NOT EXISTS PI(
PID INT PRIMARY KEY AUTO_INCREMENT,
EthMAC varchar(20) UNIQUE
)""")


	cur.execute("""CREATE TABLE IF NOT EXISTS User(
PID INT,
mail varchar(40),
alias varchar(20) UNIQUE,
FOREIGN KEY (PID) REFERENCES PI(PID)
)""")

	cur.execute("""CREATE TABLE IF NOT EXISTS Connection(
CID INT PRIMARY KEY AUTO_INCREMENT, 
PID INT, 
MAC varchar(20),
FOREIGN KEY (PID) REFERENCES PI(PID),
UNIQUE KEY (PID, MAC)
)""")

	cur.execute("""CREATE TABLE IF NOT EXISTS Central_DNS(
Domain_ID INT PRIMARY KEY AUTO_INCREMENT,
Domain varchar(50),
IP varchar(20),
UNIQUE KEY (Domain, IP)
)""")

	cur.execute("""CREATE TABLE IF NOT EXISTS Operation(
OID BIGINT PRIMARY KEY AUTO_INCREMENT,
CID INT,
Tool ENUM('Ping', 'Traceroute') NOT NULL,
StartTime TIMESTAMP,
EndTime TIMESTAMP,
Status varchar(20),
FOREIGN KEY (CID) REFERENCES Connection(CID),
UNIQUE KEY (CID, StartTime, EndTime)
)""")

	#Make sure all the tools are available!
	cur.execute("""ALTER TABLE Operation CHANGE Tool Tool ENUM('Ping', 'Traceroute', 'Httping')""")

	cur.execute("""CREATE TABLE IF NOT EXISTS Ping(
PID INT,
OID BIGINT,
SA varchar(20),
Domain_ID INT,
minRTT FLOAT,
avgRTT FLOAT,
maxRTT FLOAT,
ival INT,
size INT,
packet_loss varchar(50),
file_path varchar(50),
FOREIGN KEY (PID) REFERENCES PI(PID),
FOREIGN KEY (OID) REFERENCES Operation(OID),
FOREIGN KEY (Domain_ID) REFERENCES Central_DNS(Domain_ID),
UNIQUE KEY (OID)
)""")

	cur.execute("""CREATE TABLE IF NOT EXISTS Traceroute(
PID INT,
OID BIGINT,
SA varchar(20),
Domain_ID INT,
size INT,
file_path varchar(50),
FOREIGN KEY (PID) REFERENCES PI(PID),
FOREIGN KEY (OID) REFERENCES Operation(OID),
FOREIGN KEY (Domain_ID) REFERENCES Central_DNS(Domain_ID),
UNIQUE KEY (OID)
)""")

	cur.execute("""CREATE TABLE IF NOT EXISTS Hops(
OID BIGINT,
Hop_number INT,
Packet_number INT,
Domain_ID INT,
RTT FLOAT,
FOREIGN KEY (OID) REFERENCES Operation(OID),
FOREIGN KEY (Domain_ID) REFERENCES Central_DNS(Domain_ID),
UNIQUE KEY (OID, Hop_number, Packet_number)
)""")

	#pretty much a straight up copy of the Ping table
	cur.execute("""CREATE TABLE IF NOT EXISTS Httping(
PID INT,
OID BIGINT,
SA varchar(20),
Domain_ID INT,
minRTT FLOAT,
avgRTT FLOAT,
maxRTT FLOAT,
ival INT,
size INT,
packet_loss varchar(50),
file_path varchar(50),
FOREIGN KEY (PID) REFERENCES PI(PID),
FOREIGN KEY (OID) REFERENCES Operation(OID),
FOREIGN KEY (Domain_ID) REFERENCES Central_DNS(Domain_ID),
UNIQUE KEY (OID)
)""")
#===================================================================
	#warnings back on
	cur.execute("""SET sql_notes = 1""")
#===================================================================
##ADD SOME DATA : D
'''
	cur.execute("""INSERT INTO PI VALUES 
(NULL, 'abc'), 
(NULL, 'def')
""")

	cur.execute("""INSERT INTO User VALUES
(1, 'guy@site.com', 'PIrate'),
(2, 'person@web.se', 'PIkachu')
""")

	cur.execute("""INSERT INTO Connection VALUES 
(NULL, 1, 'abc'), 
(NULL, 2, 'def'), 
(NULL, 2, 'ghi')
""")

	cur.execute("""INSERT INTO Central_DNS VALUES 
(NULL, 'www.bbc.com', '1.2.3.4'), 
(NULL, 'www.bbc.com', '1.2.3.5'), 
(NULL, 'www.sics.se', '6.7.8.9')
""")

	cur.execute("""INSERT INTO Operation VALUES
(NULL, 1, 'Ping', NOW(), NOW(), '0'),
(NULL, 1, 'Ping', NOW(), NOW(), '1'),
(NULL, 2, 'Ping', NOW(), NOW(), '0'),
(NULL, 2, 'Traceroute', NOW(), NOW(), '4')
""")

	cur.execute("""INSERT INTO Ping VALUES
(1, 1, "193.10.66.2", 1, 25.0, 30.0, 35.0, 10, 64, "0%", "path"),
(1, 2, "193.10.66.2", 1, 24.0, 32.0, 37.0, 10, 64, "10%", "path2"),
(1, 1, "193.10.66.4", 2, 10.0, 12.0, 14.0, 10, 64, "0%", "path3")
""")
'''

con.commit()
con.close()




#!/usr/bin/env python
""" Copyright (c) 2015, Swedish Institute of Computer Science
  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
	  * Redistributions of source code must retain the above copyright
		notice, this list of conditions and the following disclaimer.
	  * Redistributions in binary form must reproduce the above copyright
		notice, this list of conditions and the following disclaimer in the
		documentation and/or other materials provided with the distribution.
	  * Neither the name of The Swedish Institute of Computer Science nor the
		names of its contributors may be used to endorse or promote products
		derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE SWEDISH INSTITUTE OF COMPUTER SCIENCE BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Authors: ljjm@sics.se
Testers:
Description: Handles all configuration file duties, including initialising
a local config file (if one does not exist, and initialising logging options
"""

from setuptools import setup, find_packages

def readme():
	with open('README.rst') as f:
		return f.read()

setup(
	name='cheesepi',
	version='0.6',
	description='CheesePi Library',
	long_description=readme(),
	url='http://cheesepi.sics.se',
	#download-url= 'http://cheesepi.sics.se/files/cheesepi.tar.gz',
	author='Liam McNamara',
	author_email='ljjm@sics.se',
	license= 'Apache 2.0',
	# Which versions we support
	classifiers=[
		'Development Status :: 4 - Beta',
		'License :: OSI Approved :: Apache Software License',
		'Operating System :: POSIX',
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 2.7',
		'Topic :: Internet',
		'Topic :: System :: Logging',
	],

	# Runtime dependencies
	install_requires=[
		'future',
		'txmsgpackrpc',
		'twisted',
		'youtube_dl',
		'dnspython',
		'influxdb',
		'pymongo',
		'uptime',
	],

	entry_points={
		'console_scripts':[
			'cheesepi = cheesepi.utils:console_script',
			'cheesepi_config = cheesepi.config:main',
			'cheesepi_control_server_start = cheesepi.server.utils:start_control_server',
			'cheesepi_upload_server_start = cheesepi.server.utils:start_upload_server',
		]
	},
	include_package_data=True,
)



import time

import logging
import pymongo
import math

from .dao import DAO
from cheesepilib.exceptions import ServerDaoError, NoSuchPeer

from cheesepilib.server.storage.models.statistics import StatisticsSet

# What is the threshold in seconds to be considered 'active'
ACTIVE_THRESHOLD = 3600

class MongoDAO(DAO):
	log = logging.getLogger("cheesepi.server.storage.MongoDAO")

	def __init__(self):
		self.__init__('localhost', 27017)

	def __init__(self, host, port):
		self.client = pymongo.MongoClient()
		self.db = self.client.cheesepi

		# Initialization stuff
		# This makes sure that the peer_id field is unique and fast lookups can
		# be performed
		self.db.beacons.create_index([("last_seen",pymongo.ASCENDING)])
		self.db.tasks.create_index([("peer_id",pymongo.ASCENDING)])
		self.db.peers.create_index([("peer_id",pymongo.ASCENDING)], unique=True)
		# This (I think) ensures fast lookups for ('peer_id','results.target_id')
		self.db.peers.create_index([("peer_id",pymongo.ASCENDING),
		                            ("results.target_id",pymongo.ASCENDING)])
		self.db.peers.create_index([("peer_id",pymongo.ASCENDING),
		                            ("statistics.target_id",pymongo.ASCENDING)])

	def close(self):
		pass # Nothing to do??

	def get_bulk_writer(self):
		return self.db.peers.initialize_ordered_bulk_op()

	def peer_beacon(self, peer_id, host, last_seen=0):
		if last_seen==0: last_seen=time.time()
		result = self.db.beacons.update_one(
			{'peer_id':peer_id},
			{'$set':{
				'peer_id':peer_id,
				'host':host,
				'last_seen':last_seen,
				}
			},
			upsert=True
		)
		return self._return_status(result.acknowledged)

	def active_peers(self, since=0):
		if since<=0: since=time.time()-ACTIVE_THRESHOLD
		result = self.db.beacons.find({"last_seen": {"$gt": since}})
		return result

	def find_peer(self, peer_id):
		result = self.db.peers.find({'peer_id':peer_id}, limit=1)
		if result.count() > 0:
			return result
		else:
			raise NoSuchPeer()

	def get_peers(self):
		result = self.db.peers.find()
		return result

	def get_random_peer(self):
		from random import randint
		num_peers = self.db.peers.count()
		#print("{} peers in collection".format(num_peers))
		result = self.db.peers.find(limit=1, skip=randint(0,num_peers))
		if result.count() > 0:
			peer = result.next()
			return peer
		else:
			# What does this mean????
			raise ServerDaoError()

	def write_task(self, peer_id, task):
		result = self.db.peers.update(
			{'peer_id':peer_id},
			{'$push': {'tasks':task}}
		)
		return result

	def get_tasks(self, peer_id):
		results = self.db.tasks.find(
			{'peer_id':peer_id},
			projection={'_id':0},
			limit=5 # should be unlimited
		)
		return results

	def register_peer(self, peer_id, host):
		result = self.db.peers.update_one(
			{'peer_id':peer_id},
			{'$set':{
				'peer_id':peer_id,
				'host':host,
				'tasks':[],
				'results':[]
				}
			},
			upsert=True
		)
		return self._return_status(result.acknowledged)

	def write_result(self, peer_id, result):
		# Can probably merge these operations into one???

		# Insert the result in the peers results-list if there is a task with a
		# matching task_id in its tasks-list
		update = self.db.peers.update(
			{
				'peer_id':peer_id,
				#'tasks': {
					#'$elemMatch':{'task_id':result['task_id']}
				#}
			},
			{'$push': {'results':result}},
			upsert=True
		)
		# Remove the task with that task_id
		#remove = self.db.peers.update(
			#{'peer_id':peer_id},
			#{'$pull': {
				#'tasks':{'task_id':result['task_id']}
				#}
			#}
		#)
		return self._return_status(update['updatedExisting'])




	def get_all_stats(self, peer_id):
		return self.get_stats_set_for_targets(peer_id, None)
	def get_stats_set(self, peer_id, target):
		"""
		Returns a StatisticsSet object loaded with all the statistics for
		measurements from peer_id to target.

		Args:
			peer_id: a peer id
			target: a Target object
		Returns:
			A StatisticsSet object if query is successful, None otherwise.
		"""
		return self.get_stats_set_for_targets(peer_id, [target])

	def get_stats_set_for_results(self, peer_id, results):
		"""
		Returns a StatisticsSet object loaded with all the statistics for
		measurements from peer_id to all targets that concern the results in
		the result list.

		Args:
			peer_id: a peer id
			results: a list of Result objects
		Returns:
			A StatisticsSet object if query is successful, None otherwise.
		"""
		targets = [result.get_target() for result in results]

		return self.get_stats_set_for_targets(peer_id, targets)

	def get_stats_set_for_targets(self, peer_id, targets):
		"""
		Returns a StatisticsSet object loaded with all the statistics for
		measurements from peer_id to all targets in the targets list.

		Args:
			peer_id: a peer id
			targets: a list of Target objects
		Returns:
			A StatisticsSet object if query is successful, None otherwise.
		"""

		projection = None

		if targets is not None:
			projection = {}
			for target in targets:
				key = "statistics.{}".format(target.get_uuid())
				projection[key] = 1

		stats = self.db.peers.find(
			{'peer_id':peer_id},
			projection,
		)

		# Should be unique so can only ever find one
		if stats.count() > 0:
			#self.log.info(stats[0])
			return StatisticsSet.fromDict(stats[0]['statistics'])
		else:
			return StatisticsSet()

	def write_stats_set(self, peer_id, statistics_set):
		"""
		Write a statistics set object to the database.

		Args:
			peer_id: a peer id
			statistcs_set: a StatisticsSet object
		Returns:
			the result from the write operation
		"""
		bulk_writer = self.db.peers.initialize_ordered_bulk_op()

		bulk_writer = self.bulk_write_stats_set(bulk_writer, peer_id,
		                                        statistics_set)

		result = bulk_writer.execute()
		self.log.info("Wrote statistics set with result: {}".format(result))
		return result

	def bulk_write_stats_set(self, bulk_writer, peer_id, statistics_set):
		"""
		Queue writing of a StatisticsSet object to a bulk writer object.

		Args:
			bulk_writer: a bulk writer object
			peer_id: a peer id
			statistcs_set: a StatisticsSet object
		Returns:
			the modified bulk writer object
		"""
		#self.log.info("IN WRITE_STATS")

		prefix_key = "statistics" #.format(target.get_uuid())

		stat_object = {}
		#self.log.info(statistics_set)

		for stat in statistics_set:
			stat_target_uuid = stat.get_target().get_uuid()
			#stat_type = stat.get_type()
			#if stat_target_uuid not in stat_object:
				#stat_object[stat_target_uuid] = {}
			#stat_object[stat_target_uuid][stat_type] = stat.toDict()

			key = "{}.{}".format(prefix_key, stat_target_uuid)

			bulk_writer.find(
				{'peer_id':peer_id}
				).upsert(
				).update_one(
				{'$set':{
					key:{
						stat.get_type():stat.toDict(),
						},
					}
				},
			)
		#self.log.info("EXITING WRITE_STATS")

		return bulk_writer

	def write_results(self, peer_id, results):
		"""
		Writes a list of results from a peer to the database.

		Args:
			peer_id: a peer id
			results: a list of Result objects
		Returns:
			the result of the write operation
		"""
		bulk_writer = self.db.peers.initialize_ordered_bulk_op()

		bulk_writer = self.bulk_write_results(bulk_writer, peer_id, results)

		result = bulk_writer.execute()
		self.log.info("Wrote a list of results objects with result: {}".format(result))
		return result

	def bulk_write_results(self, bulk_writer, peer_id, results):
		"""
		Queue writing of a list of Result objects to a bulk writer object.

		Args:
			bulk_writer: a bulk writer object
			peer_id: a peer id
			results: a list of Result objects
		Returns:
			the modified bulk writer object
		"""
		for result in results:
			bulk_writer.find(
				{'peer_id':peer_id}
				).upsert(
				).update(
				{'$push': {'results':result.toDict()}}
			)
		return bulk_writer

	def purge_results(self, peer_id):
		result = self.db.peers.update(
			{'peer_id':peer_id},
			{'results':[]}
		)
		return result

	def purge_results_older_than(self, peer_id, timestamp):
		result = self.db.peers.update(
			{'peer_id':peer_id},
			{'$pull': {
				'results':{
					'timestamp':{'$lt': timestamp}
					}
				}
			}
		)
		return result



from __future__ import print_function, unicode_literals

from builtins import str

from txmsgpackrpc.client import connect

from twisted.internet import defer

from time import time

@defer.inlineCallbacks
def callRegister(peer_id):
    c = yield connect('localhost', 18080, connectTimeout=5, waitTimeout=5)

    res = yield c.createRequest('register', peer_id)
    c.disconnect()
    defer.returnValue(res)

@defer.inlineCallbacks
def callGetTasks(peer_id):
    c = yield connect('localhost', 18080, connectTimeout=5, waitTimeout=5)
    result = yield c.createRequest('get_tasks', peer_id)
    c.disconnect()
    defer.returnValue(result)

@defer.inlineCallbacks
def callUploadResult(result_data):
    c = yield connect('localhost', 18080, connectTimeout=5, waitTimeout=5)
    res = yield c.createRequest('upload_result', result_data)
    c.disconnect()
    defer.returnValue(res)

@defer.inlineCallbacks
def cmdline(command):
    from subprocess import PIPE, Popen
    process = yield Popen(
        args=command,
        stdout=PIPE,
        shell=True
    )
    defer.returnValue(process.communicate()[0])

@defer.inlineCallbacks
def main(peer_id):
    from twisted.internet import reactor
    import time
    import os
    import re
    from random import randint

    register_response = yield callRegister(peer_id)

    while True:
        tasks_response = yield callGetTasks(peer_id)

        # If we fail, it might be because we are not registered
        if tasks_response['status'] == 'failure':
            if tasks_response['error'] == 'nosuchpeer':
                registered_response = yield callRegister(peer_id)
                tasks_response = yield callGetTasks(peer_id)
            else:
                # Something went horribly wrong
                print(tasks_response['error'])
                exit()

        tasks = tasks_response['result']
        print("got {} new tasks".format(len(tasks)))

        for task in tasks:
            task_output = yield cmdline('ping -qnc 5 ' + task['target_host'])
            #print(task_output)
            task_pattern = re.compile(r"""
                    ^rtt\ min/avg/max/mdev\ =\  # start of line
                    (\d+\.\d*)/   # min
                    (\d+\.\d*)/   # avg
                    (\d+\.\d*)/   # max
                    (\d+\.\d*)    # mdev
                    """,
                    re.MULTILINE | re.VERBOSE)
            task_match = task_pattern.search(task_output)
            #if task_match:
                #print("Matched: {}".format(task_match.groups()))
            #else:
                #print("No match...")

            time.sleep(5)
            result_data = {
                'peer_id':peer_id,
                'result':{
                    'target_id':task['target_id'],
                    'task_id':task['task_id'],
                    'task_name':'ping',
                    'value':task_match.group(2),
                    'timestamp':time.time()
                    }
            }

            print("sending result for task {}: ping to {}".format(
                task['task_id'],
                task['target_id'])
            )

            upload_response = yield callUploadResult(result_data)
            print("Upload Response: {}".format(upload_response['status']))

        time.sleep(5)

    reactor.stop()

if __name__ == "__main__":
    from twisted.internet import reactor
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('--id', type=str, default=None,
                        help='peer id')

    args = parser.parse_args()

    if args.id is None:
        exit()

    reactor.callWhenRunning(main, args.id)
    reactor.run()



from __future__ import unicode_literals, absolute_import, print_function

import logging

from .ResultParser import ResultParser

from cheesepilib.server.storage.mongo import MongoDAO
from cheesepilib.server.storage.models.result import Result

class PingResultParser(ResultParser):
	log = logging.getLogger("cheesepi.server.parsing.PingResultParser")

	# Takes an object parsed from json
	def __init__(self, obj):
		self._parsed = False
		self._input_obj = obj
		self._result_set = []
		self._peer_id = None

	def __enter__(self):
		self.parse()
		return self

	def __exit__(self, exc_type, exc_value, traceback):
		pass

	def parse(self):
		"""
		Here we should try to parse all the data we're interested in,
		and handle any resulting errors in a sane way. Should ALWAYS
		return an output that can be directly inserted into the database.
		"""
		inp = self._input_obj
		result_objects = []

		columns = inp[0]['series'][0]['columns']
		#from pprint import pformat
		#self.log.info("\n{}".format(pformat(columns)))

		entries = [entry for entry in inp[0]['series'][0]['values']]
		for entry in entries:
			# TODO THIS IS NOT IMPLEMENTED ON CLIENT SIDE, MIGHT CHANGE
			peer_id = self._peer_id = entry[columns.index('peer_id')]
			if self._peer_id is None:
				self._peer_id = peer_id
			elif self._peer_id != peer_id:
				raise Exception(
					"Found inconsistent peer_id: {}, expected: {}".format(
						peer_id, self._peer_id)
				)
			# NOTE this is done because the sequence is stored as a string
			# representation of a list, should be changed in the future so that
			# it's a list from the start
			import ast
			delay_sequence = ast.literal_eval(entry[columns.index('delays')])

			landmark = entry[columns.index('landmark')]
			target_id = entry[columns.index('target_id')]

			target = {}

			if landmark is None and target_id is not None:
				target['type'] = 'peer'
				target['ip'] = entry[columns.index('destination_address')]
				target['uuid'] = entry[columns.index('target_id')]
				target['port'] = '80' # TODO not in data
			elif landmark is not None:
				target['type'] = 'landmark'
				target['ip'] = entry[columns.index('destination_address')]
				target['domain'] = entry[columns.index('landmark')]
				target['port'] = '80' # TODO not in data

			db_entry = {
				'task_name':'ping',
				'start_time':entry[columns.index('start_time')],
				'end_time':entry[columns.index('end_time')],
				'target': target,
				'value': {
					# This is where the actual results go
					'delay_sequence':delay_sequence,
					'probe_count':entry[columns.index('ping_count')],
					'packet_loss':entry[columns.index('packet_loss')],
					'packet_size':entry[columns.index('packet_size')],
					'max_rtt':entry[columns.index('maximum_RTT')],
					'min_rtt':entry[columns.index('minimum_RTT')],
					'avg_rtt':entry[columns.index('average_RTT')],
					'stddev_rtt':entry[columns.index('stddev_RTT')],
				},
			}

			r = Result.fromDict(db_entry)
			result_objects.append(r)
			#from pprint import pformat
			#self.log.info(pformat(r.toDict()))

		self._result_objects = result_objects

		self._parsed = True

		return result_objects

	def get_peer_id(self):
		return self._peer_id



from __future__ import unicode_literals, absolute_import

import logging
import uuid

from cheesepilib.exceptions import UnsupportedTargetType

class Target(object):

	log = logging.getLogger("cheesepi.server.storage.models.target.Target")

	@classmethod
	def fromDict(cls, dct):
		from pprint import pformat
		target_type = dct['type']

		if target_type == 'landmark': return LandmarkTarget.fromDict(dct)
		if target_type == 'peer': return PeerTarget.fromDict(dct)
		else: raise UnsupportedTargetType("Unknown target type '{}'.".format(target_type))

	def toDict(self):
		raise NotImplementedError("Abstract method 'toDict' not implemented.")

	def get_uuid(self):
		raise NotImplementedError("Abstract method 'get_uuid' not implemented.")

class LandmarkTarget(Target):

	log = logging.getLogger("cheesepi.server.storage.models.target.LandmarkTarget")

	@classmethod
	def fromDict(cls, dct):
		assert dct['type'] == 'landmark'
		return LandmarkTarget(dct['ip'], dct['port'], dct['domain'])

	def __init__(self, ip, port, domain):
		self._ip = ip
		self._port = port
		self._domain = domain
		self._uuid = uuid.uuid5(uuid.NAMESPACE_DNS, domain)

	def toDict(self):
		return {
			'type':'landmark',
			'ip':self._ip,
			'port':self._port,
			'domain':self._domain,
			'uuid':str(self._uuid),
		}

	def get_uuid(self):
		return str(self._uuid)


class PeerTarget(Target):

	log = logging.getLogger("cheesepi.server.storage.models.target.PeerTarget")

	@classmethod
	def fromDict(cls, dct):
		assert dct['type'] == 'peer'
		return PeerTarget(dct['ip'], dct['port'], dct['uuid'])

	def __init__(self, ip, port, peer_uuid):
		self._ip = ip
		self._port = port
		self._uuid = uuid.UUID(peer_uuid)
		assert peer_uuid == str(self._uuid)

	def toDict(self):
		return {
			'type':'peer',
			'ip':self._ip,
			'port':self._port,
			'uuid':str(self._uuid),
		}

	def get_uuid(self):
		return str(self._uuid)
